@@ -72,6 +72,7 @@
   (set! HISTORY '())
   (set! FOUNDATION-SLOTS '())
   (set! TABLEAU-SLOTS '())
+  (set! RESERVE-SLOTS '())
   (set! EDGE-SLOTS '())
   (set! CORNER-SLOTS '())
   (set! TOP-SLOTS '())
@@ -319,6 +320,14 @@
 (define-public (find-card slot card)
   (find-card-helper card (get-cards slot) 1))
 
+(define (find-card-slot-helper slot card)
+  (if (equal? #f (find-card slot card))
+      (find-card-slot-helper (+ 1 slot) card)
+      slot))
+
+(define-public (find-card-slot card)
+  (find-card-slot-helper 0 card))
+
 ; Get the nth card from a slot. Returns #f if n is out of range.
 (define-public (get-nth-card slot-id n)
   (let ((cards (get-cards slot-id)))
@@ -459,10 +468,11 @@
                      (#t (_"the unknown card"))))
               (#t (_"the unknown card"))))))
 
-(define-public (hint-get-dest-format to-slot)
-  (if (empty-slot? to-slot)
+(define (hint-get-dest-format to-slot cards)
+  (if (null? cards)
       (cond ((member to-slot FOUNDATION-SLOTS) (if (= (length FOUNDATION-SLOTS) 1) (_"Move ~a onto the foundation.") (_"Move ~a onto an empty foundation slot.")))
             ((member to-slot TABLEAU-SLOTS) (if (= (length TABLEAU-SLOTS) 1) (_"Move ~a onto the tableau.") (_"Move ~a onto an empty tableau slot.")))
+            ((member to-slot RESERVE-SLOTS) (if (= (length RESERVE-SLOTS) 1) (_"Move ~a onto the reserve.") (_"Move ~a onto an empty reserve slot.")))
             ((member to-slot EDGE-SLOTS) (_"Move ~a onto an empty edge slot."))
             ((member to-slot CORNER-SLOTS) (_"Move ~a onto an empty corner slot."))
             ((member to-slot TOP-SLOTS) (_"Move ~a onto an empty top slot."))
@@ -470,7 +480,7 @@
             ((member to-slot LEFT-SLOTS) (_"Move ~a onto an empty left slot."))
             ((member to-slot RIGHT-SLOTS) (_"Move ~a onto an empty right slot."))
             (else (_"Move ~a onto an empty slot.")))
-      (let* ((card (get-top-card to-slot)) (value (get-value card)) (suit (get-suit card)))
+      (let* ((card (car cards)) (value (get-value card)) (suit (get-suit card)))
              (cond ((is-joker? card)
                     (if (is-black? card) (_"Move ~a onto the black joker.") (_"Move ~a onto the red joker.")))
                    ((eq? suit club) 
@@ -536,7 +546,9 @@
                    (#t (_"Move ~a onto the unknown card."))))))
 
 (define-public (hint-move from-slot from-slot-count to-slot)
-  (list 0 (format #f (hint-get-dest-format to-slot) (get-name (get-nth-card from-slot from-slot-count)))))
+  (if (= from-slot to-slot)
+      (list 0 (format #f (hint-get-dest-format to-slot (list-tail (get-cards to-slot) from-slot-count)) (get-name (get-nth-card from-slot from-slot-count))))
+      (list 0 (format #f (hint-get-dest-format to-slot (get-cards to-slot)) (get-name (get-nth-card from-slot from-slot-count))))))
 
 (define-public (hint-click slot-id hint-string)
   (list 0 hint-string))
@@ -598,6 +610,7 @@
 (define-public (set-tag! slot)
   (case (cadddr slot)
     ((tableau) (set! TABLEAU-SLOTS (cons SLOTS TABLEAU-SLOTS)))
+    ((reserve) (set! RESERVE-SLOTS (cons SLOTS RESERVE-SLOTS)))
     ((edge) (set! EDGE-SLOTS (cons SLOTS EDGE-SLOTS)))
     ((corner) (set! CORNER-SLOTS (cons SLOTS CORNER-SLOTS)))
     ((top) (set! TOP-SLOTS (cons SLOTS TOP-SLOTS)))
@@ -646,6 +659,7 @@
 (define-public IN-GAME #f)
 (define-public FOUNDATION-SLOTS '())
 (define-public TABLEAU-SLOTS '())
+(define-public RESERVE-SLOTS '())
 (define-public EDGE-SLOTS '())
 (define-public CORNER-SLOTS '())
 (define-public TOP-SLOTS '())
